@@ -20,6 +20,9 @@
 
 // Windows command-line version of eSpeak
 
+#ifndef SPEECH_H
+#define SPEECH_H
+
 
 // conditional compilation options
 
@@ -30,8 +33,9 @@
 
 #define PATHSEP '\\'
 #define USE_PORTAUDIO
-#define ESPEAK_API
-//#define INCLUDE_KLATT
+#define NO_VARIADIC_MACROS
+#define INCLUDE_KLATT
+#define INCLUDE_MBROLA
 
 // will look for espeak_data directory here, and also in user's home directory
 #define PATH_ESPEAK_DATA  "/usr/share/espeak-data"
@@ -47,7 +51,7 @@ typedef struct {
    const char *mnem;
    int  value;
 } MNEM_TAB;
-int LookupMnem(MNEM_TAB *table, char *string);
+int LookupMnem(MNEM_TAB *table, const char *string);
 
 
 
@@ -59,3 +63,4 @@ int  GetFileLength(const char *filename);
 char *Alloc(int size);
 void Free(void *ptr);
 
+#endif // SPEECH_H
