@@ -901,14 +901,21 @@ static int AnnouncePunctuation(Translator *tr, int c1, int *c2_ptr, char *output
 			else
 			if(punct_count < 4)
 			{
-				sprintf(buf,"\001+10S");
+				buf[0] = 0;
+				if(embedded_value[EMBED_S] < 300)
+					sprintf(buf,"\001+10S");  // Speak punctuation name faster, unless we are already speaking fast.  It would upset Sonic SpeedUp
+
 				while(punct_count-- > 0)
 				{
 					sprintf(buf2," %s",punctname);
 					strcat(buf, buf2);
 				}
-				sprintf(buf2," \001-10S");
-				strcat(buf, buf2);
+
+				if(embedded_value[EMBED_S] < 300)
+				{
+					sprintf(buf2," \001-10S");
+					strcat(buf, buf2);
+				}
 			}
 			else
 			{
@@ -2363,6 +2370,15 @@ f_input = f_in;  // for GetC etc
 					}
 				}
 			}
+
+			if(c1 == 0xd4d)
+			{
+				// Malayalam virama, check if next character is Zero-width-joiner
+				if(c2 == 0x200d)
+				{
+					c1 = 0xd4e;   // use this unofficial code for chillu-virama
+				}
+			}
 		}
 
 		if(iswupper(c1))
@@ -2495,7 +2511,7 @@ if(option_ssml) parag=1;
 					continue;
 				}
 
-				if((iswspace(c2) || (punct_data & 0x8000) || IsBracket(c2) || (c2=='?') || Eof()))    // don't check for '-' because if prevent recognizing ':-)'
+				if((iswspace(c2) || (punct_data & 0x8000) || IsBracket(c2) || (c2=='?') || Eof() || (c2 == ctrl_embedded)))    // don't check for '-' because it prevents recognizing ':-)'
 //				if((iswspace(c2) || (punct_data & 0x8000) || IsBracket(c2) || (c2=='?') || (c2=='-') || Eof()))
 				{
 					// note: (c2='?') is for when a smart-quote has been replaced by '?'
@@ -2675,10 +2691,11 @@ if(option_ssml) parag=1;
 		}
 		*charix_top = ix;
 
-		if(((ix > (n_buf-20)) && !IsAlpha(c1) && !iswdigit(c1))  ||  (ix >= (n_buf-2)))
+		if(((ix > (n_buf-75)) && !IsAlpha(c1) && !iswdigit(c1))  ||  (ix >= (n_buf-4)))
 		{
 			// clause too long, getting near end of buffer, so break here
 			// try to break at a word boundary (unless we actually reach the end of buffer).
+			// (n_buf-4) is to allow for 3 bytes of multibyte character plus terminator.
 			buf[ix] = ' ';
 			buf[ix+1] = 0;
 			UngetC(c2);
