@@ -200,7 +200,10 @@ int PauseLength(int pause, int control)
 	else
 		len = (pause * speed.wav_factor)/256;
 
-	if(len < 5) len = 5;      // mS, limit the amount to which pauses can be shortened
+	if(len < speed.min_pause)
+	{
+		len = speed.min_pause;      // mS, limit the amount to which pauses can be shortened
+	}
 	return(len);
 }
 
@@ -1215,6 +1218,15 @@ void DoMarker(int type, int char_posn, int length, int value)
 }  // end of DoMarker
 
 
+void DoSonicSpeed(int value)
+{//=========================
+// value, multiplier * 1024
+	wcmdq[wcmdq_tail][0] = WCMD_SONIC_SPEED;
+	wcmdq[wcmdq_tail][1] = value;
+	WcmdqInc();
+}  // end of DoSonicSpeed
+
+
 void DoVoiceChange(voice_t *v)
 {//===========================
 // allocate memory for a copy of the voice data, and free it in wavegenfill()
@@ -1381,7 +1393,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 //				DoMarker(espeakEVENT_END, count_characters, 0, count_sentences);  // end of clause
 
 			if(p->newword & 1)
-				DoMarker(espeakEVENT_WORD, sourceix, p->sourceix >> 11, clause_start_word + word_count++);
+				DoMarker(espeakEVENT_WORD, sourceix, p->sourceix >> 11, clause_start_word + word_count++);  // NOTE, this count doesn't include multiple-word pronunciations in *_list. eg (of a)
 		}
 
 		EndAmplitude();
@@ -1389,7 +1401,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 		if(p->prepause > 0)
 			DoPause(p->prepause,1);
 
-		if(option_phoneme_events && (p->type != phVOWEL))
+		if(option_phoneme_events && (p->type != phVOWEL) && (p->ph->code != phonEND_WORD))
 		{
 			// Note, for vowels, do the phoneme event after the vowel-start
 			DoMarker(espeakEVENT_PHONEME, sourceix, 0, p->ph->mnemonic);
@@ -1459,7 +1471,7 @@ int Generate(PHONEME_LIST *phoneme_list, int *n_ph, int resume)
 				}
 			}
 
-			if((prev->type==phVOWEL) || (prev->ph->phflags & phVOWEL2))
+			if((prev->type==phVOWEL) || (prev->ph->phflags & phVOWEL2) || (ph->phflags & phPREVOICE))
 			{
 				// a period of voicing before the release
 				InterpretPhoneme(NULL, 0x01, p, &phdata);
